<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Adapter;

use \Zend_Rest_Client;
use \Zend_Http_Client;
use \Magento\Framework\DataObject as DataObject;
use \FiloBlu\MappConnector\Helper\ExceptionHelper;

/**
 * Class FTPAdapter
 * @package FiloBlu\MappConnector\Adapter
 */
class FTPAdapter implements MappAdapterInterface
{
    /**
     * Client
     *
     * @var
     */
    protected $_client;

    /**
     * Rest Config
     *
     * @var
     */
    protected $_config;

    /**
     * System Config
     *
     * @var
     */
    protected $_systemconfig;

    /**
     * FileLogger Helper
     *
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var
     */
    private $_connectionId;

    /**
     * @var array
     */
    private $_allowedExtensions = ['csv'];

    /**
     * RestAdapter constructor.
     */
    public function __construct(){

    }

    /**
     * Inits the Adapter
     *
     * @throws \Zend_Http_Client_Exception
     */
    function _init(){
        $config = [
            "ftp_url" => $this->_systemconfig->getFTPUrl(),
            "ftp_login" => $this->_systemconfig->getFTPLogin(),
            "ftp_password" => $this->_systemconfig->getFTPPassword(),
            "ftp_root_dir" => $this->_systemconfig->getFTPRootDir(),
            "ftp_passive" => $this->_systemconfig->getFTPPassive(),
            "ftp_ssl" => $this->_systemconfig->getFTPSSL()
        ];

        $this->setConfig($config);
    }

    /**
     * Sets the Adapter Config
     *
     * @param $config
     */
    function setConfig($config){
        $this->_config = $config;
    }

    /**
     * Gets the Adapter Config
     *
     * @return mixed
     */
    function getConfig(){
        return $this->_config;
    }

    /**
     * Sets the System Config
     *
     * @param $config
     */
    public function setSystemConfig($config) {
        $this->_systemconfig = $config;
    }

    /**
     * Gets the System Config
     *
     * @return mixed
     */
    public function getSystemConfig() {
        return $this->_systemconfig;
    }

    /**
     * Sets the File Logger
     *
     * @param $fileLogger
     */
    public function setFileLogger($fileLogger) {
        $this->_fileLogger = $fileLogger;
    }

    /**
     * Get Magento Var Path
     *
     * @return string
     */
    protected function _getMagentoVarPath(){
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $filesystem = $objectManager->create('\Magento\Framework\Filesystem');
        $vardir = $filesystem
            ->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::VAR_DIR)
            ->getAbsolutePath();

        return $vardir . $this->_systemconfig->getLocalRootDir() . DIRECTORY_SEPARATOR . 'mapp-connector' . DIRECTORY_SEPARATOR;
    }

    /**
     * Makes the request
     *
     * @param DataObject $data
     * @return mixed
     * @throws \Exception
     */
    function makeRequest(DataObject $data){
        $localDir = $this->_getMagentoVarPath().$data['params']->config->normalized_type;
        $files = glob("$localDir/*");

        //If there is at least one file
        if(count($files)){
            $this->_fileLogger->debug('There are some file to upload ',[],__METHOD__.':'.__LINE__);

            //Connect
            if ($this->_connect()) {

                //Move to remote dir
                if($this->_moveToDir($data)){

                    //Each file will be uploaded
                    foreach($files as $file){
                        $extension = explode('.', $file);
                        if(in_array($extension[1],$this->_allowedExtensions)){
                            //Upload file in remote host
                            $this->_fileLogger->debug('Uploading File  '.$file.' ',[],__METHOD__.':'.__LINE__);
                            try {
                                if($this->_client->write(basename($file), $file)){
                                    $this->_fileLogger->debug('File '.$file.' uploaded and deleted from local path',[],__METHOD__.':'.__LINE__);
                                    //Delete file from local path
                                    unlink($file);

                                }
                                else {
                                    $this->_fileLogger->error('There was an error uploading the file '.$file,[],__METHOD__.':'.__LINE__);
                                    //Destroy FTP Connection
                                    $this->__deconstruct();
                                    return false;
                                }
                            }
                            catch (\Exception $e) {
                                $this->_fileLogger->error($e->getMessage(),[],__METHOD__.':'.__LINE__);
                                //Destroy FTP Connection
                                $this->__deconstruct();
                                return false;
                            }

                        }

                    }

                    //Destroy FTP Connection
                    $this->__deconstruct();
                    return true;

                }
                else {
                    $this->_fileLogger->error('Error creating directory',[],__METHOD__.':'.__LINE__);
                }
            }
            else {
                $this->_fileLogger->error('Failed to connect to FTP',[],__METHOD__.':'.__LINE__);
            }
        }
        else {
            $this->_fileLogger->debug('There are no file to upload for '.$data['params']->config->normalized_type,[],__METHOD__.':'.__LINE__);
        }
    }

    /**
     * FTP Connect
     *
     * @param bool $isPassive
     * @return bool
     */
    public function _connect()
    {
        try {
            $this->_fileLogger->debug('Connecting to  ' . $this->_config['ftp_url'] . ', for user ' . $this->_config['ftp_login'],[],__METHOD__.':'.__LINE__);
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $this->_client = $objectManager->create('\Magento\Framework\Filesystem\Io\Ftp');

            $open = $this->_client->open(
                array(
                    'host' => $this->_config['ftp_url'],
                    'user' => $this->_config['ftp_login'],
                    'password' => $this->_config['ftp_password'],
                    'ssl' => $this->_config['ftp_ssl'],
                    'passive' => $this->_config['ftp_passive']
                )
            );

            if($open) {
                $this->_fileLogger->debug('Connected to ' . $this->_config['ftp_url'] . ', for user ' . $this->_config['ftp_login'], [], __METHOD__ . ':' . __LINE__);
                return true;
            }
            else {
                $this->_fileLogger->error('Error connecting to ' . $this->_config['ftp_url'] . ', for user ' . $this->_config['ftp_login'],[],__METHOD__.':'.__LINE__);
                return false;
            }
        }
        catch(\Exception $e) {
            $this->_fileLogger->debug($e->getMessage(),[],__METHOD__.':'.__LINE__);
        }

    }

    /**
     * Move to remote dir
     *
     * @return bool
     */
    public function _moveToDir($data){
        $dir = '';
        if(!empty($this->_config['ftp_root_dir'])){
            $dir = $this->_config['ftp_root_dir'].DIRECTORY_SEPARATOR;
        }
        // Customers - Subscribers - Orders
        $dir .= ucfirst($data['params']->config->normalized_type).'s';

        //If Dir exists on remote server
        if($this->_client->cd($dir)){
            $this->_fileLogger->debug('Moving to directory '.$dir,[],__METHOD__.':'.__LINE__);
            return true;
        }
        //If Dir doesn't exists on remote server
        else {
            $this->_fileLogger->debug('Creating '.$dir,[],__METHOD__.':'.__LINE__);
            //Create dir on remote server
            if ($this->_client->mkdir($dir)) {
                $this->_fileLogger->debug('Directory "' . $dir . '" created successfully',[],__METHOD__.':'.__LINE__);
                //Move to dir on remote server
                $this->_client->cd($dir);
                $this->_fileLogger->debug('Moved to directory '.$dir,[],__METHOD__.':'.__LINE__);
                return true;
            } else {
                $this->_fileLogger->debug('Failed creating directory "' . $dir . '"',[],__METHOD__.':'.__LINE__);
                return false;
            }
        }
    }

    /**
     * Deconstructor
     */
    public function __deconstruct()
    {
        $this->_fileLogger->debug('Destroing FTP Connection ',[],__METHOD__.':'.__LINE__);
        $this->_client->close();
    }


}